/*********************************************************************/
/* brail.c      v 2.3      by J.Sobczyk IAPW       2001.08.02  17:01 */
/*                                                                   */
/* Polish document converter.                                        */
/* Brail encoding.                                                   */
/*-------------------------------------------------------------------*/
/* Kodowanie liter: a, -> '1', c' -> '3', e, -> '5', l- -> '2',      */
/*       n' -> '4', o' -> '0', s' -> '9', z. -> '&', z' -> '^'.      */
/* Duze litery poprzedza sie znakeim '$'.                            */
/* Ciag duzych liter poprzedza sie sekwencja "$$".                   */
/* Cyfry koduje sie: 1 -> 'A', 2 -> 'B', ... 9 -> 'I', 0 -> 'J'      */
/* poprzedzajac je znakiem: '#'.                                     */
/* Kodowanie znakow specjalnych: '.' -> '/', '!' -> '+', '(' -> '=', */
/* ')' -> '=', '"' otwierajacy -> '(', '"' zamykajacy -> ')'.        */
/* Nowa linie w wierszach oznacza sie znakiem: '@'.                  */
/* Ciagi spacji nalezy redukowac do jednej spacji.                   */
/* Ciagi pustych linii nalezy skracac do jedej pustej linii.         */
/*********************************************************************/
/* Zmainy:                                                           */
/* 1995.12.5 Ciag duzych liter poprzedza sie znakiem "$".            */
/* 1995.12.5 Ciag cyfr poprzedza sie jednym znakiem "#".             */
/*********************************************************************/
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include "polcnv.h"

/*-------------------------------------------------------------------*/
code_tbl brail_tbl = /* Brail */
  {
    { '1',   0x0105 },   /* LATIN SMALL LETTER A WITH OGONEK */
    { '3',   0x0107 },   /* LATIN SMALL LETTER C WITH ACUTE */
    { '5',   0x0119 },   /* LATIN SMALL LETTER E WITH OGONEK */
    { '2',   0x0142 },   /* LATIN SMALL LETTER L WITH STROKE */
    { '4',   0x0144 },   /* LATIN SMALL LETTER N WITH ACUTE */
    { '0',   0x00F3 },   /* LATIN SMALL LETTER O WITH ACUTE */
    { '9',   0x015B },   /* LATIN SMALL LETTER S WITH ACUTE */
    { '^',   0x017A },   /* LATIN SMALL LETTER Z WITH ACUTE */
    { '&',   0x017C },   /* LATIN SMALL LETTER Z WITH DOT ABOVE */
    { '1',   0x0104 },   /* LATIN CAPITAL LETTER A WITH OGONEK */
    { '3',   0x0106 },   /* LATIN CAPITAL LETTER C WITH ACUTE */
    { '5',   0x0118 },   /* LATIN CAPITAL LETTER E WITH OGONEK */
    { '2',   0x0141 },   /* LATIN CAPITAL LETTER L WITH STROKE */
    { '4',   0x0143 },   /* LATIN CAPITAL LETTER N WITH ACUTE */
    { '0',   0x00D3 },   /* LATIN CAPITAL LETTER O WITH ACUTE */
    { '9',   0x015A },   /* LATIN CAPITAL LETTER S WITH ACUTE */
    { '^',   0x0179 },   /* LATIN CAPITAL LETTER Z WITH ACUTE */
    { '&',   0x017B }    /* LATIN CAPITAL LETTER Z WITH DOT ABOVE */
  };

/*-------------------------------------------------------------------*/
int brail_qstr	= 0;	/* True inside of quoted string: "....."     */
int brail_level = 0;	/* Nesting level for parenthesis.            */
int brail_space	= 0;	/* True after whitespace.                    */
int brail_line	= 0;	/* Number of consecutive new line.           */
int brail_cap	= 0;	/* True after capital letter.                */
int brail_dig	= 0;	/* True after a digit.                       */

int nbrail_qstr	= 0;	/* True inside of quoted string: "....."     */
int nbrail_level	= 0;	/* Nesting level for parenthesis.            */
int nbrail_space	= 0;	/* True after whitespace.                    */
int nbrail_line	= 0;	/* Number of consecutive new line.           */
int nbrail_cap	= 0;	/* True after capital letter.                */
int nbrail_dig	= 0;	/* True after a digit.                       */
int nbrail_skip	= 0;	/* Skip this char.                           */

/*-------------------------------------------------------------------*/
long int brail_hits  = 0;
long int brail_fails = 0;

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void brail_decode( void )
  {
  int i;
  unicode ch;
  int limit;

  limit = inp_ffree;
  if (!inp_eof)
    limit--;

  while ((inp_fused < limit) &&
         (uni_ffree < uni_size ))
    {
    ch = (unicode) inp_buf[ inp_fused++ ];
    nbrail_skip		= 0;
    nbrail_space	= 0;
    nbrail_line		= 0;
    nbrail_cap		= 0;
    nbrail_dig		= 0;

    switch (ch)
      {
      case '/':
	ch = '.';
	break;
      case '+':
	ch = '!';
	break;
      case '(':
      case ')':
	ch = '"';
	break;
      case '=':
	if (brail_level == 0)
	  {
	  brail_level++;
	  ch = '(';
	  }
	else
	  {
	  brail_level--;
	  ch = ')';
	  }
	break;
      case '#':
	nbrail_dig = 1;
	if (inp_fused < inp_ffree) 
	  {
          ch = (unicode) inp_buf[ inp_fused++ ];
	  if (ch == 'J')
	    ch = '0';
	  else if ((ch >= 'A') && (ch < 'J'))
	    ch = ch - 'A' + '1';
	  else
	    ch = '#';
	  }
	break;
      case '$':
	nbrail_cap = 1;
	nbrail_skip = 1;
	break;
      default:
	if (brail_dig && ch >= 'A' && ch <= 'J')
	  {
	  if (ch == 'J')
	    i = '0';
	  else
	    i = ch - 'A' + '1';
	  ch = i;
	  nbrail_dig = 1;
	  break;
	  }
	else if (!brail_cap && ch >= 'A' && ch <= 'Z')
	  {
	  ch = ch + 'a' - 'A';
	  }
	for (i=0; i< 2*POL_NR; i++)
	  if (ch == brail_tbl[i][0])
	    {
	    if (brail_cap)
	      ch = brail_tbl[i+POL_NR][1];
	    else
	      ch = brail_tbl[i][1];
	    break;
	    }
      }

    brail_space		= nbrail_space;
    brail_line		= nbrail_line;
    brail_cap		= nbrail_cap;
    brail_dig		= nbrail_dig;

    if (!nbrail_skip)
      uni_buf[ uni_ffree++ ] = ch;
    }

  if ((inp_ffree == inp_fused) && inp_eof)
    uni_eof = TRUE;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void brail_encode( void )
  {
  int i;
  unicode ch;

  while ((uni_ffree > uni_fused) &&
         (out_ffree < out_size-1))
    {
    ch = uni_buf[uni_fused++];

    nbrail_skip		= 0;
    nbrail_space	= 0;
    nbrail_line		= 0;
    nbrail_cap		= 0;
    nbrail_dig		= 0;

    for (i=0; i< 2*POL_NR; i++)
      if (ch == brail_tbl[i][1])
	{
	if (i >= POL_NR)
	  {
	  if (!brail_cap)
	    out_buf[out_ffree++] = '$';
	  nbrail_cap	= 1;
	  }
	ch = brail_tbl[i][0];
	break;
        }

    if (i >= 2*POL_NR)
      {
      switch (ch)
	{
	case '.':
	  ch = '/';
	  break;
	case '!':
	  ch = '+';
	  break;
	case '"':
	  if (brail_qstr)
	    {
	    ch = ')';
	    brail_qstr = 0;
	    }
	  else
	    {
	    ch = '(';
	    brail_qstr = 1;
	    }
	  break;
	case '(':
	case ')':
	  ch = '=';
	  break;
	case ' ':
	case '\t':
	  nbrail_space = 1;
	  if (!brail_space)
	    ch = ' ';
	  else
	    nbrail_skip = 1;
	  break;
	case '\n':
	  nbrail_line = brail_line + 1;
	  if (brail_line < 2)
	    ch = '\n';
	  else
	    nbrail_skip = 1;
	  break;
	default:
	  if (ch >= 'A' && ch <= 'Z' && !brail_cap)
	    {
	    out_buf[out_ffree++] = '$';
	    nbrail_cap = 1;
	    }
	  else if (ch >= 'A' && ch <= 'Z')
	    {
	    nbrail_cap = 1;
	    }
	  else if (ch >= '0' && ch <= '9')
	    {
	    if (!brail_dig)
	      {
	      out_buf[out_ffree++] = '#';
	      }
	    if (ch == '0')
	      i = 'J';
	    else
	      i = ch - '1' + 'A';
	    ch = i;
	    nbrail_dig = 1;
	    }
	  else
	    ch = ch;
	}
      }

    brail_space		= nbrail_space;
    brail_line		= nbrail_line;
    brail_cap		= nbrail_cap;
    brail_dig		= nbrail_dig;

    if (!nbrail_skip)
      out_buf[out_ffree++] = (byte) ch;
    }

  if ((uni_ffree == uni_fused) && uni_eof)
    out_eof = TRUE;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void test_char_brail( byte *buff )
  {
  int i;

  if (buff[0] & 0x80)
    {
    brail_fails++;
    return;
    }

  for (i=0; i< 2*POL_NR; i++)
    if (  (  ((byte) buff[0] == brail_tbl[i][0]) && isalpha(buff[1]) ) ||
          (  ((byte) buff[1] == brail_tbl[i][0]) && isalpha(buff[0]) ) )
      {
      brail_hits++;
      return;
      }
    else if (  ((byte) buff[0] == brail_tbl[i][0]) ||
               ((byte) buff[1] == brail_tbl[i][0]) )
      {
      brail_fails++;
      return;
      }

  if ((buff[0] == '#') && (tolower(buff[1]) >= 'a') &&
			  (tolower(buff[1]) <= 'j')    )
    {
    brail_hits++;
    return;
    }

  brail_fails++;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
int test_freq_brail( void )
  {
  return brail_hits-brail_fails;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void init_brail( void )
  {
  trans[ TYPE_BRAIL ].key		= (char*)"-b";
  trans[ TYPE_BRAIL ].long_key		= (char*)"-brail";
  trans[ TYPE_BRAIL ].name		= (char*)"Brail";
  trans[ TYPE_BRAIL ].reset_inp	= NULL;;
  trans[ TYPE_BRAIL ].reset_out	= NULL;;
  trans[ TYPE_BRAIL ].decode		= brail_decode;
  trans[ TYPE_BRAIL ].encode		= brail_encode;
  trans[ TYPE_BRAIL ].inp_tbl		= NULL;
  trans[ TYPE_BRAIL ].inp_tbl_size	= 0;
  trans[ TYPE_BRAIL ].out_tbl		= NULL;
  trans[ TYPE_BRAIL ].out_tbl_size	= 0;
  trans[ TYPE_BRAIL ].buf_tst		= NULL;
  trans[ TYPE_BRAIL ].freq_tst	= test_freq_brail;
  }

/*********************************************************************/
