/*********************************************************************/
/* file.c       v 2.3      by J.Sobczyk IAPW       2001.08.02  17:01 */
/*                                                                   */
/* Polish document converter.                                        */
/* User defined conversion.                                          */
/*********************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include "polcnv.h"

/*-------------------------------------------------------------------*/
tbl_ptr file_tbl = NULL;	/* Internal use only! */
int     file_tbl_size = 0;

/*-------------------------------------------------------------------*/
struct
  {
  char code[3];
  unicode key;
  } user_codes[2*POL_NR] =
  {
  { "a,", 0x0105 },   /* LATIN SMALL LETTER A WITH OGONEK */
  { "c'", 0x0107 },   /* LATIN SMALL LETTER C WITH ACUTE */
  { "e,", 0x0119 },   /* LATIN SMALL LETTER E WITH OGONEK */
  { "l-", 0x0142 },   /* LATIN SMALL LETTER L WITH STROKE */
  { "n'", 0x0144 },   /* LATIN SMALL LETTER N WITH ACUTE */
  { "o'", 0x00F3 },   /* LATIN SMALL LETTER O WITH ACUTE */
  { "s'", 0x015B },   /* LATIN SMALL LETTER S WITH ACUTE */
  { "z'", 0x017A },   /* LATIN SMALL LETTER Z WITH ACUTE */
  { "z.", 0x017C },   /* LATIN SMALL LETTER Z WITH DOT ABOVE */
  { "A,", 0x0104 },   /* LATIN CAPITAL LETTER A WITH OGONEK */
  { "C'", 0x0106 },   /* LATIN CAPITAL LETTER C WITH ACUTE */
  { "E,", 0x0118 },   /* LATIN CAPITAL LETTER E WITH OGONEK */
  { "L-", 0x0141 },   /* LATIN CAPITAL LETTER L WITH STROKE */
  { "N'", 0x0143 },   /* LATIN CAPITAL LETTER N WITH ACUTE */
  { "O'", 0x00D3 },   /* LATIN CAPITAL LETTER O WITH ACUTE */
  { "S'", 0x015A },   /* LATIN CAPITAL LETTER S WITH ACUTE */
  { "Z'", 0x0179 },   /* LATIN CAPITAL LETTER Z WITH ACUTE */
  { "Z.", 0x017B }    /* LATIN CAPITAL LETTER Z WITH DOT ABOVE */
  };

/*-------------------------------------------------------------------*/
/* Find next field separated by sep. Simmilar to strtok, except that */
/* Uses single character delimiter and allows empty fields.          */
/*-------------------------------------------------------------------*/
char *strfield_buff = NULL;
char *strfield_end  = NULL;
char  strfield_char;

char *strfield( char *buff, char sep )
  {
  char *res;

  if (buff != NULL)
    {
    strfield_buff = buff;
    strfield_end  = buff + strlen( buff );
    }
  else
    {
    strfield_buff += strlen( strfield_buff );
    if (strfield_buff == strfield_end)
      return NULL;

    *strfield_buff++ = strfield_char;
    }
  
  res = strchr( strfield_buff, sep );
  if (res != NULL)
    {
    strfield_char = *res;
    *res = '\0';
    }

  return strfield_buff;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void load_table( char *table )
  {
  static char line[200];
  FILE *inp;
  int i;
  char *val;
  unicode code;
  unicode key;
  int ffree = 0;
  file_tbl_size = 0;
  
  /* ---- Count lines ---- */
  inp = fopen( table, "r" );
  if (inp == NULL)
    {
    fprintf( stderr, "Can not open table definition = %s\n\a", table );
    exit( 1 );
    }

  while (fgets( line, 200, inp ) != NULL)
    file_tbl_size++;
  fclose( inp );

  file_tbl = (tbl_ptr) calloc( 2*file_tbl_size, sizeof(unicode) );
  if (file_tbl == NULL)
    {
    fprintf( stderr, "Can not allocate memory for conversion table\n\a" );
    exit( 1 );
    }

  /* ---- Read the table ---- */
  inp = fopen( table, "r" );
  if (inp == NULL)
    {
    fprintf( stderr, "Can not open table definition %s\n\a", table );
    exit( 1 );
    }

  while (fgets( line, 200, inp ) != NULL)
    {
    /* ---- remove all comments ---- */
    val = strchr( line, '#');
    if (val != NULL)
      *val = '\0';

    if (strchr( line, '=')==NULL)
      {	/* ---- Unicode mapping file ---- */
      /* ------ code '\t' key ----------- */
      sscanf( line, "%lx %lx", &code, &key );

      file_tbl[ffree][0] = code;
      file_tbl[ffree][1] = key;
      ffree++;
      }
    else
      { /* ---- Shortcut convention ---- */
      /* ------ key '=' code ----------- */
      for (i=0; line[i] != '\0'; i++)
	if (isspace(line[i]))
	  {
	  strcpy( line+i, line+i+1 );
	  i--;
	  }
	else if (line[i] == '#')
	  {
	  line[i] = '\0';
	  i--;
	  }
	  
      val = strtok( line, "=" );
      val = strtok( NULL, "=" );
      if (val == NULL)
	{
	fprintf( stderr, "No value for keyword: %s - skipping\n", line );
	continue;
	}
      code = (unicode) strtol( val, NULL, 0 );

      for (i=0; i<2*POL_NR; i++)
	if (!strcmp( user_codes[i].code, line ))
	  break;
	  
      if (i < 2*POL_NR)
	{
	file_tbl[ffree][0] = code;
	file_tbl[ffree][1] = user_codes[i].key;
	ffree++;
	}
      else
	fprintf( stderr, "Unknown keyword: %s - ignored\n", line );
      }
    }

  fclose( inp );
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void dump_short_table( void )
  {
  int i, j;

  if (decode != table_decode)
    fprintf( stderr, "Not a single character code!\n" );

  for (i=0; i < 2*POL_NR; i++)
    {
    printf( "%s = ", user_codes[i].code );
    for (j=0; j < inp_tbl_size; j++)
      if (inp_tbl[j][1] == user_codes[i].key)
	break;
    if (j < inp_tbl_size)
      printf( "0x%0lX # %ld %c\n",
	    inp_tbl[j][0],
		  inp_tbl[j][0],
		      (unsigned char) (inp_tbl[j][0] & 0x0FFL) );
    else
      printf( "0x%0lX # %ld %c\n",
	    user_codes[i].key,
		  user_codes[i].key,
		      (unsigned char) (user_codes[i].key & 0x0FFL) );
    }
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void dump_full_table( void )
  {
  int j;
  int i;
  static char line[200];
  FILE *inp;
  char *val;
  unicode key;

  inp = fopen( "UnicodeData.txt", "r" );
  if (inp == NULL)
    { /* Simpy dump all numbers */
    if (decode != table_decode)
      fprintf( stderr, "Not a single character code!\n" );

    for (j=0; j < inp_tbl_size; j++)
      printf( "0x%0lX\t0x%0lX\t# \n",
	      inp_tbl[j][0],
		    inp_tbl[j][1] );
    return;
    }

  while (fgets( line, 200, inp ) != NULL)
    {
    val = strfield( line, ';' );
    key = (unicode) strtol( line, NULL, 16 );
    for (j=0; j < inp_tbl_size; j++)
      if (inp_tbl[j][1] == key)
	break;
    if (j < inp_tbl_size)
      {
      printf( "0x%0lX\t0x%0lX\t# ",
	      inp_tbl[j][0],
		    inp_tbl[j][1] );
      val = strfield( NULL, ';' );
      if (val != NULL)
	printf( "%s", val );
      for (i=1; i<10; i++)		/* Skip to the 10th field */
	val = strfield( NULL, ';' );
      if (val != NULL)
	printf( " ; %s\n", val );
      else
	printf( "\n" );
      }
    }
  fclose( inp );

  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void set_file( char *table, char *file, int sel )
  {
  switch (sel)
    {
    case 0:   inp_name	= file;
	      inp_type	= TYPE_FILE;
	      decode	= table_decode;
	      load_table( table );
	      inp_tbl    = file_tbl;
	      inp_tbl_size = file_tbl_size;
	      break;
    case 1:   out_name	= file;
	      out_type	= TYPE_FILE;
	      encode	= table_encode;
	      load_table( table );
	      out_tbl    = file_tbl;
	      out_tbl_size = file_tbl_size;
	      break;
    default:  fprintf( stderr, "Too many switches\a\n" );
	      help();
    }
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
int test_file( void )
  {
  return table_test( file_tbl, file_tbl_size );
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void init_file( void )
  {
  trans[ TYPE_FILE ].key		= (char*)"-f file";
  trans[ TYPE_FILE ].name		= (char*)"user defined";
  trans[ TYPE_FILE ].reset_inp		= NULL;
  trans[ TYPE_FILE ].reset_out		= NULL;
  trans[ TYPE_FILE ].decode		= table_decode;
  trans[ TYPE_FILE ].encode		= table_encode;
  trans[ TYPE_FILE ].inp_tbl		= NULL;
  trans[ TYPE_FILE ].inp_tbl_size	= 0;
  trans[ TYPE_FILE ].out_tbl		= NULL;
  trans[ TYPE_FILE ].out_tbl_size	= 0;
  trans[ TYPE_FILE ].buf_tst		= NULL;
  trans[ TYPE_FILE ].freq_tst		= NULL;
  }

/*********************************************************************/
