/*********************************************************************/
/* html.c     v 2.3      by J.Sobczyk IAPW       2001.08.02  17:01   */
/*                                                                   */
/* Polish document converter.                                        */
/* HTML notation                                                     */
/*********************************************************************/
#include <stdio.h>
#include <ctype.h>
#include "polcnv.h"

/*-------------------------------------------------------------------*/
#define HTML_TBL_SIZE 96
#define HTML_TAG_LEN 10
typedef struct 
  {
  char tag[HTML_TAG_LEN];
  unicode code;
  } str_code;
typedef str_code str_code_tbl[HTML_TBL_SIZE];

str_code_tbl html_tbl = /* Latin 1 ISO 8859-1 */
  {
    { "nbsp",    0x0a0 }, /* no-break space  */
    { "iexcl",   0x0a1 }, /* inverted exclamation mark  */
    { "cent",    0x0a2 }, /* cent sign  */
    { "pound",   0x0a3 }, /* pound sterling sign  */
    { "curren",  0x0a4 }, /* general currency sign  */
    { "yen",     0x0a5 }, /* yen sign  */
    { "brvbar",  0x0a6 }, /* broken (vertical) bar  */
    { "sect",    0x0a7 }, /* section sign  */
    { "uml",     0x0a8 }, /* umlaut (dieresis)  */
    { "copy",    0x0a9 }, /* copyright sign  */
    { "ordf",    0x0aa }, /* ordinal indicator, feminine  */
    { "laquo",   0x0ab }, /* angle quotation mark, left  */
    { "not",     0x0ac }, /* not sign  */
    { "shy",     0x0ad }, /* soft hyphen  */
    { "reg",     0x0ae }, /* registered sign  */
    { "macr",    0x0af }, /* macron  */
    { "deg",     0x0b0 }, /* degree sign  */
    { "plusmn",  0x0b1 }, /* plus-or-minus sign  */
    { "sup2",    0x0b2 }, /* superscript two  */
    { "sup3",    0x0b3 }, /* superscript three  */
    { "acute",   0x0b4 }, /* acute accent  */
    { "micro",   0x0b5 }, /* micro sign  */
    { "para",    0x0b6 }, /* pilcrow (paragraph sign)  */
    { "middot",  0x0b7 }, /* middle dot  */
    { "cedil",   0x0b8 }, /* cedilla  */
    { "sup1",    0x0b9 }, /* superscript one  */
    { "ordm",    0x0ba }, /* ordinal indicator, masculine  */
    { "raquo",   0x0bb }, /* angle quotation mark, right  */
    { "frac14",  0x0bc }, /* fraction one-quarter  */
    { "frac12",  0x0bd }, /* fraction one-half  */
    { "frac34",  0x0be }, /* fraction three-quarters  */
    { "iquest",  0x0bf }, /* inverted question mark  */
    { "Agrave",  0x0c0 }, /* capital A, grave accent  */
    { "Aacute",  0x0c1 }, /* capital A, acute accent  */
    { "Acirc",   0x0c2 }, /* capital A, circumflex accent  */
    { "Atilde",  0x0c3 }, /* capital A, tilde  */
    { "Auml",    0x0c4 }, /* capital A, dieresis or umlaut mark  */
    { "Aring",   0x0c5 }, /* capital A, ring  */
    { "AElig",   0x0c6 }, /* capital AE diphthong (ligature)  */
    { "Ccedil",  0x0c7 }, /* capital C, cedilla  */
    { "Egrave",  0x0c8 }, /* capital E, grave accent  */
    { "Eacute",  0x0c9 }, /* capital E, acute accent  */
    { "Ecirc",   0x0ca }, /* capital E, circumflex accent  */
    { "Euml",    0x0cb }, /* capital E, dieresis or umlaut mark  */
    { "Igrave",  0x0cc }, /* capital I, grave accent  */
    { "Iacute",  0x0cd }, /* capital I, acute accent  */
    { "Icirc",   0x0ce }, /* capital I, circumflex accent  */
    { "Iuml",    0x0cf }, /* capital I, dieresis or umlaut mark  */
    { "ETH",     0x0d0 }, /* capital Eth, Icelandic  */
    { "Ntilde",  0x0d1 }, /* capital N, tilde  */
    { "Ograve",  0x0d2 }, /* capital O, grave accent  */
    { "Oacute",  0x0d3 }, /* capital O, acute accent  */
    { "Ocirc",   0x0d4 }, /* capital O, circumflex accent  */
    { "Otilde",  0x0d5 }, /* capital O, tilde  */
    { "Ouml",    0x0d6 }, /* capital O, dieresis or umlaut mark  */
    { "times",   0x0d7 }, /* multiply sign  */
    { "Oslash",  0x0d8 }, /* capital O, slash  */
    { "Ugrave",  0x0d9 }, /* capital U, grave accent  */
    { "Uacute",  0x0da }, /* capital U, acute accent  */
    { "Ucirc",   0x0db }, /* capital U, circumflex accent  */
    { "Uuml",    0x0dc }, /* capital U, dieresis or umlaut mark  */
    { "Yacute",  0x0dd }, /* capital Y, acute accent  */
    { "THORN",   0x0de }, /* capital THORN, Icelandic  */
    { "szlig",   0x0df }, /* small sharp s, German (sz ligature)  */
    { "agrave",  0x0e0 }, /* small a, grave accent  */
    { "aacute",  0x0e1 }, /* small a, acute accent  */
    { "acirc",   0x0e2 }, /* small a, circumflex accent  */
    { "atilde",  0x0e3 }, /* small a, tilde  */
    { "auml",    0x0e4 }, /* small a, dieresis or umlaut mark  */
    { "aring",   0x0e5 }, /* small a, ring  */
    { "aelig",   0x0e6 }, /* small ae diphthong (ligature)  */
    { "ccedil",  0x0e7 }, /* small c, cedilla  */
    { "egrave",  0x0e8 }, /* small e, grave accent  */
    { "eacute",  0x0e9 }, /* small e, acute accent  */
    { "ecirc",   0x0ea }, /* small e, circumflex accent  */
    { "euml",    0x0eb }, /* small e, dieresis or umlaut mark  */
    { "igrave",  0x0ec }, /* small i, grave accent  */
    { "iacute",  0x0ed }, /* small i, acute accent  */
    { "icirc",   0x0ee }, /* small i, circumflex accent  */
    { "iuml",    0x0ef }, /* small i, dieresis or umlaut mark  */
    { "eth",     0x0f0 }, /* small eth, Icelandic  */
    { "ntilde",  0x0f1 }, /* small n, tilde  */
    { "ograve",  0x0f2 }, /* small o, grave accent  */
    { "oacute",  0x0f3 }, /* small o, acute accent  */
    { "ocirc",   0x0f4 }, /* small o, circumflex accent  */
    { "otilde",  0x0f5 }, /* small o, tilde  */
    { "ouml",    0x0f6 }, /* small o, dieresis or umlaut mark  */
    { "divide",  0x0f7 }, /* divide sign  */
    { "oslash",  0x0f8 }, /* small o, slash  */
    { "ugrave",  0x0f9 }, /* small u, grave accent  */
    { "uacute",  0x0fa }, /* small u, acute accent  */
    { "ucirc",   0x0fb }, /* small u, circumflex accent  */
    { "uuml",    0x0fc }, /* small u, dieresis or umlaut mark  */
    { "yacute",  0x0fd }, /* small y, acute accent  */
    { "thorn",   0x0fe }, /* small thorn, Icelandic  */
    { "yuml",    0x0ff }, /* small y, dieresis or umlaut mark  */
  };

long int hfreq[ 256 ];

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void html_decode( void )
  {
  int i, k;
  unicode ch;
 
  while ((inp_fused < inp_ffree) &&
         (uni_ffree < uni_size-HTML_TAG_LEN ))
    {
    ch = (unicode) inp_buf[ inp_fused++ ];

    if (ch == '&')
      {
      if (inp_buf[ inp_fused ] == '#')
	{
	ch = 0;
	inp_fused++;
	while (inp_buf[ inp_fused ] >= '0' && inp_buf[ inp_fused ] <= '9')
	  ch = ch * 10 + inp_buf[ inp_fused++ ] - '0';
	if (inp_buf[inp_fused] == ';')
	  inp_fused++;
	}
      else
	{
	for (i=0; i<inp_tbl_size; i++)
	  {
	  for (k=0; html_tbl[i].tag[k] != '\0'; k++)
	    if (inp_buf[inp_fused+k] != html_tbl[i].tag[k])
	      break;

	  if( html_tbl[i].tag[k] == '\0')
	    {
	    ch = html_tbl[i].code;
	    inp_fused += k;
	    if (inp_buf[inp_fused] == ';')
	      inp_fused++;
	    break;
	    }
	  }
	}
      }

    uni_buf[ uni_ffree++ ] = ch;
    }
 
  if ((inp_ffree == inp_fused) && inp_eof)
    uni_eof = TRUE;
  }
 
/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void html_encode( void )
  {
  int i, k;
  unicode ch;
 
  while ((uni_ffree > uni_fused) &&
         (out_ffree < out_size-HTML_TAG_LEN))	/* Reserve ten places in the  */
    {					/* buffer for the html     */
    ch = uni_buf[uni_fused++];
 
    for (i=0; i<out_tbl_size; i++)
      if (ch == html_tbl[i].code)
	{
        printf( "%lx \n", ch ); /* DEB */
	out_buf[out_ffree++] = (byte) '&';
	for (k=0; html_tbl[i].tag[k] != '\0'; k++)
	  out_buf[out_ffree++] = (byte) html_tbl[i].tag[k];
        ch = (unicode) ';';
	break;
	}
 
    out_buf[out_ffree++] = (byte) ch;
    }
        printf( "\n" ); /* DEB */
 
  if ((uni_ffree == uni_fused) && uni_eof)
    out_eof = TRUE;
  }
 
/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void test_buf_html( void )
  {
  /****
  int i;

  for (i=0; i< 2*POL_NR; i++)
    if (buff[1] == html_tbl[i])
      hfreq[ buff[0] & 0xFF ]++;
  *******/
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
int test_freq_html( void )
  {
  int best;

  best    = -1;
  return best;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void set_html( char html, char *file, int mode )
  {
  switch (mode)
    {
    case 0:   inp_name		= file;
	      inp_type		= TYPE_HTML;
	      inp_tbl		= NULL;
	      inp_tbl_size	= 2*POL_NR;
	      decode		= html_decode;
	      break;
    case 1:   out_name		= file;
	      out_type		= TYPE_HTML;
	      out_tbl		= NULL;
	      out_tbl_size	= 2*POL_NR;
	      encode		= html_encode;
	      break;
    default:  fprintf( stderr, "Too many switches\a\n" );
	      help();
    }
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void init_html( void )
  {
  int i;

  for (i=0; i<256; i++)
    hfreq[i] = 0;

  trans[ TYPE_HTML ].key		= (char*)"-html";
  trans[ TYPE_HTML ].long_key		= (char*)"-html";
  trans[ TYPE_HTML ].name		= (char*)"HTML notation";
  trans[ TYPE_HTML ].reset_inp		= NULL;
  trans[ TYPE_HTML ].reset_out		= NULL;
  trans[ TYPE_HTML ].encode		= html_encode;
  trans[ TYPE_HTML ].decode		= html_decode;
  trans[ TYPE_HTML ].inp_tbl		= NULL; /* Exception - accessed indirectly */
  trans[ TYPE_HTML ].inp_tbl_size	= HTML_TBL_SIZE;
  trans[ TYPE_HTML ].out_tbl		= NULL; /* Exception - accessed indirectly */
  trans[ TYPE_HTML ].out_tbl_size	= HTML_TBL_SIZE;
  trans[ TYPE_HTML ].buf_tst		= test_buf_html;
  trans[ TYPE_HTML ].freq_tst		= test_freq_html;
  }

/*********************************************************************/
