/*********************************************************************/
/* polcnv.c     v 2.3      by J.Sobczyk IAPW       2001.08.02  17:01 */
/*                                                                   */
/* Polish document converter.                                        */
/*********************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "polcnv.h"
#ifdef SVR4
#define stricmp( S1, S2 ) strcasecmp( S1, S2 )
#define strnicmp( S1, S2, LEN ) strncasecmp( S1, S2, LEN )
#endif

/*-------------------------------------------------------------------*/
int  inp_type   = TYPE_UNSPECIFIED;
byte inp_prefix = '\0';
char *inp_name  = (char *) NULL;
FILE *inp_file  = (FILE *) NULL;
void (*decode)( void );
tbl_ptr inp_tbl;
int  inp_tbl_size;

int  inp_ffree  = 0;
int  inp_fused  = 0;
int  inp_size   = BUF_SIZE;
int  inp_eof    = FALSE;
/*byte inp_buf[ BUF_SIZE + BUF_ZONE ];*/
byte *inp_buf;

/*-------------------------------------------------------------------*/
int  uni_ffree  = 0;
int  uni_fused  = 0;
int  uni_size   = BUF_SIZE;
int  uni_eof    = FALSE;
/*unicode uni_buf[ BUF_SIZE + BUF_ZONE ];*/
unicode *uni_buf;

/*-------------------------------------------------------------------*/
int  out_type   = TYPE_UNSPECIFIED;
byte out_prefix = '\0';
char *out_name  = (char *) NULL;
FILE *out_file  = (FILE *) NULL;
void (*encode)( void );
tbl_ptr out_tbl;
int  out_tbl_size;

int  out_ffree  = 0;
int  out_fused  = 0;
int  out_size   = BUF_SIZE;
int  out_eof    = FALSE;
/*byte out_buf[ BUF_SIZE + BUF_ZONE ];*/
byte *out_buf;

/*-------------------------------------------------------------------*/
trans_table trans[ MAX_TRANS ];
int verbose_flag = 0;
int dump_flag = 0;

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void shift( void )
  {
  int i;

  /* ------ Shift input buffer ------ */
  for (i=0; i<inp_ffree-inp_fused; i++)
    inp_buf[i] = inp_buf[ inp_fused + i ];

  inp_ffree -= inp_fused;
  inp_fused = 0;

  /* ------ Shift Unicode buffer ------ */
  for (i=0; i<uni_ffree-uni_fused; i++)
    uni_buf[i] = uni_buf[ uni_fused + i ];

  uni_ffree -= uni_fused;
  uni_fused = 0;

  /* ------ Shift output buffer ------ */
  for (i=0; i<out_ffree-out_fused; i++)
    out_buf[i] = out_buf[ out_fused + i ];

  out_ffree -= out_fused;
  out_fused = 0;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void padd( void )
  {
  int i;

  /* ------ Padd input buffer ------ */
  for (i=inp_ffree; i<BUF_SIZE+BUF_ZONE; i++)
    inp_buf[i] = 0;

  /* ------ Padd Unicode buffer ------ */
  for (i=uni_ffree; i<BUF_SIZE+BUF_ZONE; i++)
    uni_buf[i] = 0;

  /* ------ Padd output buffer ------ */
  for (i=out_ffree; i<BUF_SIZE+BUF_ZONE; i++)
    out_buf[i] = 0;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void readfile( void )
  {
  int bsize, rsize;

  if (inp_eof)
    return;

  if (inp_ffree > inp_size)
    return;

  bsize = inp_size - inp_ffree;
  rsize = fread( &inp_buf[ inp_ffree ], 1, bsize, inp_file );

  if (rsize < bsize)
    inp_eof = TRUE;

  inp_ffree += rsize;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void writefile( void )
  {
  if (out_ffree > out_fused)
    fwrite( &out_buf[ out_fused ], 1, out_ffree - out_fused, out_file );

  out_fused = out_ffree;
  }

/*-------------------------------------------------------------------*/
void convert( void )
  {
  if (verbose_flag > 1)
    {
    if (inp_type == TYPE_PREFIX)
      fprintf( stderr, "Converting from %s using %c ",
				 trans[ inp_type ].name, inp_prefix );
    else
      fprintf( stderr, "Converting from %s ", trans[ inp_type ].name );
    if (out_type == TYPE_PREFIX)
      fprintf( stderr, "to %s using %c ",
				 trans[ out_type ].name, out_prefix );
    else
      fprintf( stderr, "to %s\n", trans[ out_type ].name );
    }

  if (!strcmp(inp_name, "-"))
    inp_file = stdin;
  else
    {
    inp_file = fopen( inp_name, "r" );
    if (inp_file == NULL)
      {
      fprintf( stderr, "Can not open input file: %s\a\n", inp_name );
      exit( 2 );
      }
    }

  if (!strcmp(out_name, "-"))
    out_file = stdout;
  else
    {
    out_file = fopen( out_name, "w" );
    if (out_file == NULL)
      {
      fprintf( stderr, "Can not open output file: %s\a\n", out_name );
      exit( 2 );
      }
    }

  while (!out_eof)
    {
    shift();
    readfile();
    padd();
    decode();
    encode();
    writefile();
    }
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void help( void )
  {
  int t;

  printf( "POLcnv  v2.2  by J.Sobczyk " __DATE__ "\n" );

  printf( "\n"
          "Usage:\n"
	  "    POLcnv [options] inputtype inputfile outputtype outputfile\n"
	  "or:\n"
	  "    POLcnv [options] -t inputfile\n"
	  "Types:\n"
	  "                 -     -p@  - prefix notation using @\n"
	);

  for (t=0; t<MAX_TRANS; t++)
    if (t != TYPE_PREFIX)
      printf( "%15s  - %7s  - %s\n", trans[t].long_key, trans[t].key, trans[t].name );

  printf( "Options:\n"
	  "    -?   - Display this help\n"
	  "    -d   - Dump input conversion table\n"
	  "    -v   - Increase verbosity level\n"
	  "Hyphen used in place of file specification denotes\n"
	  "standard input or output stream.\n"
	  "Use: POLcnv -v -? for more explanations.\n"
	);

  if (verbose_flag > 0)
    {
    printf( "Examples:\n"
	    "  POLcnv -m aaa.txt -i bbb.doc\n"
	    "    convert file aaa.txt coded using Mazowia to bbb.doc using"
	                                                      "ISO 8859-2\n"
	    "  POLcnv -f my.cod abc.txt -i def.txt\n"
	    "    Convert file abc.txt from code defined with my.cod to ISO 8859-2\n"
	    "  POLcnv -d -i\n"
	    "    Dump ISO 8859-2 code table\n"
	    "  POLcnv -t aaa.txt -m ccc.txt\n"
	    "    identify code used for file aaa.txt and covert it to Mazovia\n"
	    "  POLcnv -t xxx.txt\n"
	    "    identify code used for coding file xxx.txt\n"
	    "  POLcnv -v -v -t xxx.txt\n"
	    "    identify code used for coding file xxx.txt and display gathered"
								  " statistics\n"
	    "Verbosity levels:\n"
	    "  0 - error reports only, report only best result for -t\n"
	    "  1 - all positive test results\n"
	    "  2 - dump non zero frequency table entries for character codes"
								  " 0x80..0xff\n"
	    "      and prefix frequencies for non alphabetic and non space"
	    " characters\n"
	    "  3 - dump all non zero frequency table entries and prefix frequencies"
	    " for all characters\n"
	  );
    }

  exit( 1 );
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void init( void )
  {
  int i;
 
  inp_buf = (byte *)	calloc( BUF_SIZE + BUF_ZONE, sizeof(byte) );
  uni_buf = (unicode *)	calloc( BUF_SIZE + BUF_ZONE, sizeof(unicode) );
  out_buf = (byte *)	calloc( BUF_SIZE + BUF_ZONE, sizeof(byte) );
  if ((inp_buf == NULL) || (uni_buf == NULL) || (out_buf == NULL))
    {
    fprintf( stderr, "No memory!\a\n" );
    exit(1);
    }

/*byte out_buf[ BUF_SIZE + BUF_ZONE ];*/
/*unicode uni_buf[ BUF_SIZE + BUF_ZONE ];*/
/*byte inp_buf[ BUF_SIZE + BUF_ZONE ];*/

  for (i=0; i < MAX_TRANS; i++)
    {
    trans[ i ].key		= (char*)"";
    trans[ i ].long_key		= (char*)"";
    trans[ i ].name		= (char*)"";
    trans[ i ].reset_inp	= NULL;
    trans[ i ].reset_out	= NULL;
    trans[ i ].encode		= NULL;
    trans[ i ].decode		= NULL;
    trans[ i ].inp_tbl		= NULL;
    trans[ i ].inp_tbl_size	= 0;
    trans[ i ].out_tbl		= NULL;
    trans[ i ].out_tbl_size	= 0;
    trans[ i ].inp_ffree	= 0;
    trans[ i ].inp_fused	= 0;
    trans[ i ].buf_tst		= NULL;
    trans[ i ].freq_tst		= NULL;
    }
  init_test();
  init_file();
  init_prefix();
  init_iso_8859_2();
  init_iso_8859_1();
  init_mazowia();
  init_latin_2();
  init_windows();
  init_microvex();
  init_unia();
  init_macintosh();

  init_csk();
  init_cyfromat();
  init_dhn();
  init_iinte();
  init_swierk();
  init_logic();
  init_ventura();
  init_elwro();
  init_amiga();
  init_texpl();
  init_atari();
  init_corel();
  init_atm();
  init_bez();

  init_brail();
  init_iso_6937_2();
  init_unicode_ucs4();
  init_unicode_ucs2();
  init_unicode_utf8();
  init_unicode_utf7();
  init_unicode_java();
  init_hex();
  init_html();
/*************
  init_unicode();
  init_iso_10646();
*************/
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void set_trans( int nr, char *file, int mode )
  {
  switch (mode)
    {
    case 0:   inp_name		= file;
	      inp_type		= nr;
	      decode		= trans[ nr ].decode;
	      inp_tbl		= trans[ nr ].inp_tbl;
	      inp_tbl_size	= trans[ nr ].inp_tbl_size;
	      if (trans[ nr ].reset_inp != NULL)
	        (*trans[ nr ].reset_inp)();
	      break;
    case 1:   out_name		= file;
	      out_type		= nr;
	      encode		= trans[ nr ].encode;
	      out_tbl		= trans[ nr ].out_tbl;
	      out_tbl_size	= trans[ nr ].out_tbl_size;
	      if (trans[ nr ].reset_out != NULL)
	        (*trans[ nr ].reset_out)();
	      break;
    default:  fprintf( stderr, "Too many switches\a\n" );
	      help();
    }

  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
int main( int narg, char *argv[] )
  {
  int i;
  int t;
  int mode = 0;

  init();

  for (i=1; i<narg; i++)
    {
    if      (!stricmp( argv[i], "-?" ))
      help();
    else if (!stricmp( argv[i], "-v" ))
      verbose_flag++;
    else if (!stricmp( argv[i], "-dd" ))
      dump_flag += 2;
    else if (!stricmp( argv[i], "-d" ))
      dump_flag++;
    else if (!strnicmp( argv[i], "-p", 2 ))
      {
      set_prefix( argv[i][2], argv[i+1], mode++ );
      i++;
      }
    else if (!stricmp( argv[i], "-f" ))
      {
      set_file( argv[i+1], argv[i+2], mode++ );
      i+=2;
      }
    else if (!strnicmp( argv[i], "-f=", 3 ))
      {
      set_file( argv[i]+3, argv[i+1], mode++ );
      i+=1;
      }
    else
      {
      for (t=0; t<MAX_TRANS; t++)
	{
	if (!stricmp( argv[i], trans[t].key ) || !stricmp( argv[i], trans[t].long_key ))
	  {
	  set_trans( t, argv[i+1], mode++ );
	  i++;
	  break;
	  }
	}
      if (t == MAX_TRANS)
	{
	fprintf( stderr, "invalid switch: %s\a\n", argv[i] );
	help();
	}
      }
    }

  if (dump_flag > 1)
    {
    dump_full_table();
    exit( 0 );
    }
  else if (dump_flag)
    {
    dump_short_table();
    exit( 0 );
    }

  if (inp_name == NULL)
    {
    fprintf( stderr, "No input file specified\a\n" );
    help();
    }

  if (inp_type == TYPE_TEST)
    run_test();

  if (out_name == NULL)
    exit( 1 );

  if ((inp_type != TYPE_TEST) && (inp_type != TYPE_UNSPECIFIED))
    convert();
  }

/*********************************************************************/
