/*********************************************************************/
/* ucs2.c    v 2.3      by J.Sobczyk IAPW       2001.08.02  17:01    */
/*                                                                   */
/* Polish document converter.                                        */
/* Unicode (ISO 10646) UCS2 == UTF-16 ?? as described in RFC 2044    */
/*********************************************************************/
#include <stdio.h>
#include "polcnv.h"

long int ucs2_hits 	= 0;
long int ucs2_fails	= 0;
int      ucs2_0nbsp	= TRUE;
int      ucs2_first	= TRUE;
int      ucs2_reverse	= FALSE;

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void ucs2_decode( void )
  {
  int limit;
  int i;
  unicode ch1, ch2;

  limit = inp_ffree;
  if (!inp_eof)
    limit = ((int)(limit / 2)) * 2;

  if (limit % 2 != 0)
    {
    fprintf( stderr, "Uncomplete character " );
    for (i=((int)(limit/2))*2; i<limit; i++)
      fprintf( stderr, "%02x", inp_buf[ i ] );
    fprintf( stderr, " at the end of file - padded with 0's\n" );

    /* Fill remainig bytes with 0's */
    for (i=limit; i<((int)((limit+1)/2))*2; i++)
      inp_buf[ i ]= 0;
    }

  if (ucs2_first)
    {			/* Detect byte order - RFC 2070		     */
    if (inp_buf[inp_fused] == 0x0FF && inp_buf[inp_fused+1] == 0x0FE)
      {
      fprintf( stderr, "Low-high byte order - reversing!\n" );
      inp_fused += 2;
      ucs2_reverse = TRUE;
      }
    else if (inp_buf[inp_fused] == 0x0FE && inp_buf[inp_fused+1] == 0x0FF)
      {
      inp_fused += 2;
      }
    ucs2_first = FALSE;
    }

  while ((inp_fused < limit) &&
	 (uni_ffree < uni_size ))
    {
    if (!ucs2_reverse)
      {
      if ((inp_buf[inp_fused]   >= 0x0D8) &&
	  (inp_buf[inp_fused]   <  0x0DC) &&
	  (inp_buf[inp_fused+2] >= 0x0DC) &&
	  (inp_buf[inp_fused+2] <  0x0DF)   )
	{
	ch1 = 
	  ((unicode)(inp_buf[ inp_fused++ ]) << 8) +
	   (unicode)(inp_buf[ inp_fused++ ]);
	ch2 = 
	  ((unicode)(inp_buf[ inp_fused++ ]) << 8) +
	   (unicode)(inp_buf[ inp_fused++ ]);
	uni_buf[ uni_ffree++ ] =
	  ((ch1 - 0x0D800UL) << 10) + (ch2 - 0x0DC00UL) + 0x10000UL;
	}
      else
	uni_buf[ uni_ffree++ ] =
	  ((unicode)(inp_buf[ inp_fused++ ]) << 8) +
	   (unicode)(inp_buf[ inp_fused++ ]);
      }
    else
      {		/* Reversed byte order 				     */
      if ((inp_buf[inp_fused+1] >= 0x0D8) &&
	  (inp_buf[inp_fused+1] <  0x0DC) &&
	  (inp_buf[inp_fused+3] >= 0x0DC) &&
	  (inp_buf[inp_fused+3] <  0x0DF)   )
	{
	ch1 = 
	   (unicode)(inp_buf[ inp_fused++ ]) +
	  ((unicode)(inp_buf[ inp_fused++ ]) << 8);
	ch2 = 
	   (unicode)(inp_buf[ inp_fused++ ]) +
	  ((unicode)(inp_buf[ inp_fused++ ]) << 8);
	uni_buf[ uni_ffree++ ] =
	  ((ch1 - 0x0D800UL) << 10) + (ch2 - 0x0DC00UL) + 0x10000UL;
	}
      else
	uni_buf[ uni_ffree++ ] =
	   (unicode)(inp_buf[ inp_fused++ ]) +
	  ((unicode)(inp_buf[ inp_fused++ ]) << 8);
      }
    }

  if ((inp_ffree == inp_fused) && inp_eof)
    uni_eof = TRUE;
  }


/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void ucs2_encode( void )
  {
  unicode ch;
  unicode res;

  if (ucs2_0nbsp)
    {		
    if (uni_buf[uni_fused] != 0x0FEFFUL)
      {		/* Always start with ZERO-WIDTH NON-BREAKING SPACE   */
		  /* as recommended in RFC2070			     */
      out_buf[out_ffree++] = 0x0FE;
      out_buf[out_ffree++] = 0x0FF;
      }
    ucs2_0nbsp = FALSE;
    }

  while ((uni_ffree > uni_fused) &&
	 (out_ffree < out_size-1))
    {
    if (uni_buf[uni_fused] < 0x0FFFFUL)
      {
      out_buf[out_ffree++] = (byte) ((uni_buf[uni_fused] >> 8)  & 0x0FF);
      out_buf[out_ffree++] = (byte) ( uni_buf[uni_fused]        & 0x0FF);
      }
    else if (uni_buf[uni_fused] > 0x10FFFFUL)
      {
      out_buf[out_ffree++] = 0x0FF;
      out_buf[out_ffree++] = 0x0FD;
      }
    else
      {
      ch = uni_buf[uni_fused] - 0x10000UL;
      res = (ch >> 10) + 0x0D800UL;
      out_buf[out_ffree++] = (byte) ((res >> 8)  & 0x0FF);
      out_buf[out_ffree++] = (byte) ( res        & 0x0FF);
      res = (ch & 0x03FFUL) + 0x0DC00UL;
      out_buf[out_ffree++] = (byte) ((res >> 8)  & 0x0FF);
      out_buf[out_ffree++] = (byte) ( res        & 0x0FF);
      }

    uni_fused++;
    }

  if ((uni_ffree == uni_fused) && uni_eof)
    out_eof = TRUE;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void test_char_ucs2( byte *buff )
  {
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
int test_freq_ucs2( void )
  {
  return ucs2_hits-ucs2_fails;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void ucs2_reset_inp( void )
  {
  ucs2_0nbsp	= TRUE;
  ucs2_first	= TRUE;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void ucs2_reset_out( void )
  {
  ucs2_0nbsp	= TRUE;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void init_unicode_ucs2( void )
  {
  ucs2_0nbsp	= TRUE;
  ucs2_first	= TRUE;
  ucs2_reverse	= FALSE;

  trans[ TYPE_UNICODE_UCS2 ].key		= (char*)"-u2";
  trans[ TYPE_UNICODE_UCS2 ].long_key		= (char*)"-ucs-2";
  trans[ TYPE_UNICODE_UCS2 ].name		= (char*)"Unicode UCS-2";
  trans[ TYPE_UNICODE_UCS2 ].reset_inp		= ucs2_reset_inp;
  trans[ TYPE_UNICODE_UCS2 ].reset_out		= ucs2_reset_inp;
  trans[ TYPE_UNICODE_UCS2 ].decode		= ucs2_decode;
  trans[ TYPE_UNICODE_UCS2 ].encode		= ucs2_encode;
  trans[ TYPE_UNICODE_UCS2 ].inp_tbl		= NULL;
  trans[ TYPE_UNICODE_UCS2 ].inp_tbl_size	= 0;
  trans[ TYPE_UNICODE_UCS2 ].out_tbl		= NULL;
  trans[ TYPE_UNICODE_UCS2 ].out_tbl_size	= 0;
  trans[ TYPE_UNICODE_UCS2 ].buf_tst		= NULL;
  trans[ TYPE_UNICODE_UCS2 ].freq_tst		= test_freq_ucs2;
  }

/*********************************************************************/
